/*
** Copyright 2001, Michael Pfeiffer	
*/

#include <KernelExport.h>
#include <Drivers.h>
#include <Errors.h>
#include <FindDirectory.h>
#include <image.h>
#include <stdio.h>
#include <string.h>
#include <graphic_driver.h>
#include <unistd.h>
#include "../include/AccelerantInterface.h"
#include "../include/DriverInterface.h"

/* begin refresh rate hack */
static struct {
	int width;
	int height;
} modes[] = {
	{  640, 480},
	{  800, 600},
	{ 1024, 768},
	{ 1152, 768},
	{ 1280, 1024},
	{ 1600, 1200}
};

status_t hack_display_mode(display_mode *mode_to_set) {
	status_t err;
	int      i;
	int      rf;
	bool     enabled = is_hertz_enabled();

#ifdef DEBUG
	char buffer[255];

	sprintf(buffer, "%dx%d/%dHz\n", mode_to_set->timing.h_display, mode_to_set->timing.v_display,
		(int)(mode_to_set->timing.pixel_clock * 1000 / mode_to_set->timing.h_total / 
		mode_to_set->timing.v_total));
	LOG(buffer);
#endif

	if ((!enabled) || mode_to_set == NULL) return B_ERROR;
	
	// search full screen display mode
	for (i = 0; i < 6; i++) {
		if (mode_to_set->timing.h_display == modes[i].width &&
		    mode_to_set->timing.v_display == modes[i].height) break;
	}
	
	// calculate current refresh rate
	rf = ( 1000 * (long)mode_to_set->timing.pixel_clock )/ 
		 ( mode_to_set->timing.h_total * (long)mode_to_set->timing.v_total );	
	
	
	if (i < NO_OF_FULLSCREEN_MODES && 59 <= rf && rf <= 61) {
		int hz;
		// change mode	
		display_mode mode = *mode_to_set;
		// get refresh rate for mode "i"
		hz = get_fullscreen_refreshrate((enum full_screen_mode) i);
		{
#ifdef DEBUG
			LOG("GDW_FULLSCREEN_REFRESHRATE return error\n");
			sprintf(buffer, "hz = %d.%1.1d\n", (int)hz/10, (int)hz%10);
			LOG(buffer);
#endif
			hz = 600;
		}
		if (hz >= 600 && hz <= 1000) {
			// change pixel clock
			mode.timing.pixel_clock = mode.timing.h_total * mode.timing.v_total * hz / 10000;
			err = A_SET_DISPLAY_MODE(&mode);
			if (err == B_OK) { 
				LOG("fullscreen mode set\n"); 
				return B_OK; 
			} else {
				LOG("error could not set fullscreen mode\n");
			}
		}
	}
	return B_ERROR; 
}
/* end refresh rate hack */

