/*
** Copyright 2001, Michael Pfeiffer	
*/

#include <image.h>
#include <be/storage/FindDirectory.h>
#include <be/support/String.h>
#include <be/storage/Path.h>
#include <be/storage/Entry.h>
#include <be/storage/Directory.h>
#include "../include/DriverInterface.h"
#include "../include/AccelerantInterface.h"
#include "../include/TVOutInterface.h"

/* global variables */
static image_id tv_out_image    = -1;
static status_t (*g_load_tv_out)(int driver)   = NULL;
static void     (*g_unload_tv_out)(void) = NULL;
static status_t (*g_set_tv_out_mode)(display_mode *mode_to_set, 
                             enum tv_out_mode mode,
                             status_t (*SET_DISPLAY_MODE)(display_mode *mode_to_set)) = NULL;

static bool          tv_out_active = false;
static display_mode  requested_mode;

static bool load(int driver, directory_which which) {
	BPath  path;
	BEntry entry;
	BDirectory dir;
	
	if (B_OK == find_directory(which, &path) &&
	    B_OK == path.Append(TV_OUT_ADD_ON_PATH) &&
	    B_OK == entry.SetTo(path.Path()) &&
	    B_OK == dir.SetTo(&entry)) {

		BEntry cur;
		while (B_OK == dir.GetNextEntry(&cur)) {
			int32 *api_version;
			BPath path;
			cur.GetPath(&path);
			LOG(path.Path()); LOG("\n");
			// load add on
			image_id id = load_add_on(path.Path());
			if (id <= 0) continue; // or id < 0???
			bool ok = B_OK == get_image_symbol(id, "load_tv_out", B_SYMBOL_TYPE_TEXT, (void**)&g_load_tv_out);
			ok = ok && B_OK == get_image_symbol(id, "unload_tv_out", B_SYMBOL_TYPE_TEXT, (void**)&g_unload_tv_out);
			ok = ok && B_OK == get_image_symbol(id, "set_tv_out_mode", B_SYMBOL_TYPE_TEXT, (void**)&g_set_tv_out_mode);
			ok = ok && B_OK == get_image_symbol(id, "api_version", B_SYMBOL_TYPE_DATA, (void**)&api_version);
			if (ok && *api_version == TV_OUT_ADDON_API_VERSION && B_OK == g_load_tv_out(driver)) {
				tv_out_image = id;
				LOG("load returns with true\n");
				return true;						
			}
			unload_add_on(id);
		}		
	}
	LOG("load returns with false\n");
	return false;
}

extern "C" void load_tv_out_add_on(int driver) {
	LOG("load_tv_out_add_on\n"); 
    if (load(driver, B_USER_ADDONS_DIRECTORY) ||
	    load(driver, B_BEOS_ADDONS_DIRECTORY)) {
	}
}

extern "C" void unload_tv_out_add_on() {
	if (tv_out_image != -1) {
		g_unload_tv_out();
		unload_add_on(tv_out_image);
		tv_out_image = -1;
	} 
}

#include <stdio.h>
// determine tv_out_mode by refresh rate
static enum tv_out_mode get_mode(display_mode* mode_to_set) {
	// calculate current refresh rate in 1/10 Hz 
	int rf = ( 10000 * (long)mode_to_set->timing.pixel_clock  + 9999)/ 
		 ( mode_to_set->timing.h_total * (long)mode_to_set->timing.v_total );	

#ifdef DEBUG
	{
	char buffer[80];
	sprintf(buffer, "tvout refresh rate= %d.%d\n", rf/10, rf%10);
	LOG(buffer); 
	}
#endif

	if (rf == get_tvout_refreshrate(TV_OUT_MODE)) return TV_OUT_MODE;
	else if (rf == get_tvout_refreshrate(DUAL_MODE)) return DUAL_MODE;
	return VGA_MODE;
}

extern "C" status_t tv_out_set_display_mode(display_mode *mode_to_set,
                                            status_t (*set_display_mode)(display_mode*)) {
	if (tv_out_image == -1) return B_ERROR;
	// disable tv-out mode
	if (tv_out_active) {
		g_set_tv_out_mode(mode_to_set, VGA_MODE, set_display_mode);
		tv_out_active = false;
	}
	enum tv_out_mode mode = get_mode(mode_to_set);
	if (mode != VGA_MODE && is_tvout_enabled()) {
		// tv-out mode requested, try to set it: 
		requested_mode = *mode_to_set;
		status_t err = g_set_tv_out_mode(mode_to_set, mode, set_display_mode);
		if (err == B_OK) tv_out_active = true;
		return err;
	}
	return B_ERROR;	
}

extern "C" status_t tv_out_get_display_mode(display_mode *mode) {
	if (tv_out_image == -1) return B_ERROR;

	if (tv_out_active) {
		// pretend that the requested mode is set:
		*mode = requested_mode;
		return B_OK;
	}

	return B_ERROR;	
}

